import json
import os
from pathlib import Path

def get_config(current_dir: Path = None, return_path=False):
    if current_dir is None:
        current_dir = Path(os.getcwd())

    config_path = current_dir / ".yatlogger.json"
    if config_path.exists():
        with config_path.open() as config_file:
            config = json.load(config_file)
            if "token" not in config:
                raise KeyError("Key 'token' is missing in config file")

            config["users"] = config.get("users", [])

            if return_path:
                return config, config_path
            return config

    if current_dir.parent == current_dir:
        raise FileNotFoundError("Could not find config")

    return get_config(current_dir.parent, return_path)



from pathlib import Path
from argparse import ArgumentParser
from .register import run_register_service

if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("--config")
    args = parser.parse_args()

    config_search_dir = None if args.config is None else Path(args.config)

    run_register_service(config_search_dir)



import json
import random
from pathlib import Path
from telegram import Update
from telegram.ext import MessageHandler, CommandHandler, CallbackContext, Updater, Filters
from .config import get_config


DIGITS = 6


def run_register_service(config_search_dir: Path = None):
    config, config_path = get_config(config_search_dir, return_path=True)
    print("Using config file", config_path)

    code = random.randint(0, sum([9 * (10 ** i) for i in range(DIGITS)]))
    code = str(code).zfill(DIGITS)
    print(f"Enter this code on Telegram to register the chat to the bot:\n{code}\n")

    def _start(update: Update, context: CallbackContext):
        context.bot.send_message(
            chat_id=update.effective_chat.id,
            text="Hello, if you want to register, send me the passcode as a message."
        )

    def _msg(update: Update, context: CallbackContext):
        if update.message.text == code:
            chat_id = update.effective_chat.id
            if chat_id in config["users"]:
                context.bot.send_message(
                    chat_id=chat_id,
                    text="Hey, I know you already!",
                )
            else:
                config["users"].append(chat_id)
                with open(config_path, "w") as f:
                    json.dump(config, f)

                print("Registered", chat_id)

                context.bot.send_message(
                    chat_id=update.effective_chat.id,
                    text=f"""Succesfully registered this chat.
Note that currently running instances will not log messages to this chat.
You can stop the register process on the logging machine now.""",
                )

    bot = Updater(token=config["token"], use_context=True)
    bot.dispatcher.add_handler(CommandHandler('start', _start))
    bot.dispatcher.add_handler(MessageHandler(Filters.text & (~Filters.command), _msg))

    print("Press Ctrl+C to stop the bot")

    bot.start_polling()
    bot.idle()



import sys
import logging
from logging import LogRecord
from pathlib import Path
from typing import Union
import telegram
from telegram.utils.helpers import escape_markdown
from .config import get_config


__version__ = '0.1.1'


class TelegramHandler(logging.Handler):
    "A logging handler that sends messages to Telegram"
    def __init__(self, session_name: str = None, level=logging.INFO,
                 config: Union[str, Path, dict] = None):
        """Create a new logging handler that sends messages to Telegram.

        `session_name` is a text that will be prepended to messages to be able
        to better identify the source of the message.
        
        `level` is the minimum logging level that the handler will react to.

        `config` can either by a directory or a `dict`.
        If `config` is a directory, the handler will try to find a file named
        `.yatlogger.json` in that directory or starting from there in any of
        the parent directories. If `config` is a `dict`, the handler will use
        it directly. If `config` is `None`, the handler will look for a config
        file starting from the current working directory.
        """
        super().__init__(level=level)

        self.session_name = session_name

        if config is None:
            config = get_config()
        elif not isinstance(config, dict):
            config = get_config(config)

        self.bot = telegram.Bot(token=config["token"])
        self.chat_ids = config["users"]

    def emit(self, record: LogRecord):
        message = escape_markdown(self.format(record), version=2)
        session_name = None
        if self.session_name is not None:
            session_name = escape_markdown(self.session_name, version=2)

        prepend = ""
        if record.levelno == logging.ERROR:
            if session_name is None:
                prepend = "*Error*\n"
            else:
                prepend = f"*Error in {session_name}*\n"
        elif session_name is not None:
            prepend = f"_{session_name}_\n"

        for chat_id in self.chat_ids:
            self.bot.send_message(chat_id=chat_id, text=prepend + message,
                                  parse_mode="MarkdownV2")


def register(session_name: str = None, logger_name: str = None,
             level=logging.INFO, config: Union[str, Path, dict] = None):
    """Helper function that calls both `register_logger` and
    `hook_exception_log`.
    
    The used logger will be returned."""
    logger = register_logger(session_name, logger_name, level, config)
    hook_exception_log(logger_name)
    return logger


def register_logger(session_name: str = None, logger_name: str = None,
                    level=logging.INFO, config: Union[str, Path, dict] = None):
    """Helper function to create a new `TelegramHandler` and adding it the
    logger with name `name`.

    The `level` and `config` arguments are passed to `TelegramHandler`.

    The used logger will be returned.
    """
    logger = logging.getLogger(name=logger_name)
    logger.addHandler(TelegramHandler(session_name=session_name, level=level,
                                      config=config))
    return logger


def hook_exception_log(logger_name: str = None):
    """Sets an except hook to `sys.excepthook` that forwards the exception
    value to `logger.error()`.
    `KeboardInterrupt` won't be logged."""
    logger = logging.getLogger(name=logger_name)

    def custom_hook(type, value, traceback):
        if not isinstance(type, KeyboardInterrupt):
            logger.error(value)
        sys.__excepthook__(type, value, traceback)
        
    sys.excepthook = custom_hook


class log_exceptions:
    """A context manager that logs unhandled exceptions to the given logger.

    ``` python
    with yatlogger.log_exceptions():
        try:
            # this error will not be logged
            raise ValueError("handled eror")
        except:
            pass

        # this error will be logged using logging.error()
        raise ValueError("unhandled error")
    ```
    """
    def __init__(self, logger_name: str = None):
        """Create a new `log_exception` context manager."""
        self.logger = logging.getLogger(logger_name)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        if exc_type is None or isinstance(exc_type, KeyboardInterrupt):
            return

        self.logger.error(exc_value)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yatlogger']

package_data = \
{'': ['*']}

install_requires = \
['python-telegram-bot>=13.6,<14.0']

setup_kwargs = {
    'name': 'yatlogger',
    'version': '0.1.1',
    'description': 'Yet another telegram logger',
    'long_description': '# Yet Another Telegram Logger\n\n## Setup\n\n### 1. Create a bot\n\nFirst, [create a new bot](https://core.telegram.org/bots#creating-a-new-bot). It\'s basically sending some messages to [@BotFather](https://t.me/botfather).\n\n### 2. Create a config file (`.yatlogger.json`)\n\nNext, create a file named `.yatlogger.json` and place it in the same directory as your code or in a one of the parent directories. The file must look like this:\n\n``` json\n{\n    "token": "<your api key>"\n}\n```\n\nReplace `<your api key>` with the API key you got from the BotFather.\n\n### 3. Register chats\n\nYour bot must know to which chats it should send the logs. So the next step is to register receiving chats.\n\nRun `python -m yatlogger` to start the register service. As long as this service is running, you can register new chats.\n\nTo register a chat, start a chat with your bot and enter the 6 digit pin you see on the logging machine.\n\n![register a new chat](./docs/register_chat.jpg)\n\nWhen you are done, you can simply interrupt the register service with <kbd>Ctrl</kbd> + <kbd>C</kbd>\n\n## Usage\n\nyatlogger registers itself as a handler for the `logging` module. Here is an example:\n\n``` python\nimport logging\nimport yatlogger\n\nlogger = yatlogger.register()\nlogger.setLevel(logging.INFO)\n\nlogger.info("Read this on your phone!")\n\nraise ValueError("This unhandled exception will be sent to Telegram, too!")\n\n```\n\n![log messages on telegram](./docs/logs.jpg)\n',
    'author': 'cyd3r',
    'author_email': 'cyd3rhacker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/cyd3r/yatlogger',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
